@@ -13,10 +13,9 @@ import pika
 from flask import Flask
 from flask import jsonify
 
-from biomaj_download.download.ftp import FTPDownload
-from biomaj_download.download.http import HTTPDownload
+from biomaj_download.download.curl import CurlDownload
 from biomaj_download.download.direct import DirectFTPDownload
-from biomaj_download.download.direct import DirectHttpDownload
+from biomaj_download.download.direct import DirectHTTPDownload
 from biomaj_download.download.localcopy import LocalDownload
 from biomaj_download.message import downmessage_pb2
 from biomaj_download.download.rsync import RSYNCDownload
@@ -28,6 +27,14 @@ app = Flask(__name__)
 app_log = logging.getLogger('werkzeug')
 app_log.setLevel(logging.ERROR)
 
+# Classify protocols from downmessage.proto
+# Note: those lists are based on the protocol numbers, not the protocol names
+ALL_PROTOCOLS = [item for key, item in downmessage_pb2.DownloadFile.Protocol.items()]
+DIRECT_PROTOCOLS = [
+    item for key, item in downmessage_pb2.DownloadFile.Protocol.items()
+    if key.startswith("DIRECT")
+]
+
 
 @app.route('/api/download-message')
 def ping():
@@ -122,33 +129,36 @@ class DownloadService(object):
     def get_handler(self, protocol_name, server, remote_dir, remote_files=[],
                     credentials=None, http_parse=None, http_method=None, param=None,
                     proxy=None, proxy_auth='',
-                    save_as=None, timeout_download=None, offline_dir=None):
+                    save_as=None, timeout_download=None, offline_dir=None,
+                    protocol_options={}):
         protocol = downmessage_pb2.DownloadFile.Protocol.Value(protocol_name.upper())
         downloader = None
-        if protocol in [0, 1]:
-            downloader = FTPDownload(protocol_name, server, remote_dir)
-        if protocol in [2, 3]:
-            downloader = HTTPDownload(protocol_name, server, remote_dir, http_parse)
-        if protocol == 7:
+        if protocol in [0, 1]:  # FTP, SFTP
+            downloader = CurlDownload(protocol_name, server, remote_dir)
+        if protocol in [2, 3]:  # HTTP, HTTPS (could be factored with previous case)
+            downloader = CurlDownload(protocol_name, server, remote_dir, http_parse)
+        if protocol == 4:  # DirectFTP
+            downloader = DirectFTPDownload("ftp", server, '/')
+        if protocol == 5:  # DirectHTTP
+            downloader = DirectHTTPDownload("http", server, '/')
+        if protocol == 6:  # DirectHTTPS
+            downloader = DirectHTTPDownload("https", server, '/')
+        if protocol == 10:  # DirectFTPS
+            downloader = DirectFTPDownload('ftps', server, '/')
+        if protocol == 7:  # Local
             downloader = LocalDownload(remote_dir)
-        if protocol == 4:
-            downloader = DirectFTPDownload('ftp', server, '/')
-        if protocol == 5:
-            downloader = DirectHttpDownload('http', server, '/')
-        if protocol == 6:
-            downloader = DirectHttpDownload('https', server, '/')
-        if protocol == 8:
-            downloader = RSYNCDownload('rsync', server, remote_dir)
-        if protocol == 9:
-            downloader = IRODSDownload('irods', server, remote_dir)
+        if protocol == 8:  # RSYNC
+            downloader = RSYNCDownload(server, remote_dir)
+        if protocol == 9:  # iRods
+            downloader = IRODSDownload(server, remote_dir)
         if downloader is None:
             return None
 
         for remote_file in remote_files:
             if remote_file['save_as']:
                 save_as = remote_file['save_as']
-        # For direct protocol, we only keep base name
-        if protocol in [4, 5, 6]:
+        # For direct protocols, we only keep base name
+        if protocol in DIRECT_PROTOCOLS:
             tmp_remote = []
             for remote_file in remote_files:
                 tmp_remote.append(remote_file['name'])
@@ -171,13 +181,19 @@ class DownloadService(object):
 
         if save_as:
             downloader.set_save_as(save_as)
+
         if param:
             downloader.set_param(param)
 
         downloader.set_server(server)
 
+        # Set the name of the BioMAJ protocol to which we respond.
         downloader.set_protocol(protocol_name)
 
+        if protocol_options is not None:
+            self.logger.debug("Received protocol options: " + str(protocol_options))
+            downloader.set_options(protocol_options)
+
         downloader.logger = self.logger
         downloader.set_files_to_download(remote_files)
         return downloader
@@ -226,7 +242,9 @@ class DownloadService(object):
                                 proxy_auth=proxy_auth,
                                 save_as=biomaj_file_info.remote_file.save_as,
                                 timeout_download=biomaj_file_info.timeout_download,
-                                offline_dir=biomaj_file_info.local_dir)
+                                offline_dir=biomaj_file_info.local_dir,
+                                protocol_options=biomaj_file_info.protocol_options
+                                )
 
     def clean(self, biomaj_file_info=None):
         '''
